package com.example.todo;

import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;

import org.apache.http.*;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.DefaultHttpClient;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.app.Activity;
import android.content.ContentValues;
import android.content.Intent;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.util.Log;

public class RemoteTodoSync extends Activity {
  public static final String GETADDRESS = "http://10.0.0.4/test.php";
  public static final String PUTADDRESS = "http://10.0.0.4/test2.php";
  private static final String TAG = "RemoteTodoSync";
  
  private String[] localProjection = new String[] {
      TodoDatabaseProvider.COL_TASK,
      TodoDatabaseProvider.COL_DUEDATE,
      TodoDatabaseProvider.COL_CREATEDATE,
      TodoDatabaseProvider.COL_CATEGORY_LINK
  };    

  private JSONArray taskArray = new JSONArray();  
  
  @Override
  public void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);
    getData();
    addTodosFromServer(taskArray);
    getLocalTasks();
    sendTasksToRemoteDb();
    Intent i = new Intent();
    setResult(RESULT_OK, i);
    finish(); 
  }  
    
  private void getData() {
    InputStream is = null;
    try {
      HttpClient httpClient = new DefaultHttpClient();
      HttpPost httpPost = new HttpPost(GETADDRESS);
      HttpResponse response = httpClient.execute(httpPost);
      HttpEntity entity = response.getEntity();
      is = entity.getContent();
    } catch (IOException e) {
      Log.e(TAG, "IOException " + e.toString());
      e.printStackTrace();
    }
    try {
      BufferedReader reader = new BufferedReader
          (new InputStreamReader(is, "iso-8859-1"), 8);
      String line = null;
      while ((line = reader.readLine()) != null) {
        taskArray = new JSONArray(line);
      }
      is.close();
    } catch (Exception e) {
      Log.e(TAG, "Exception: " + e.toString());
      e.printStackTrace();
    }
  }
    
  private void addTodosFromServer(JSONArray todoDataFromServer) {
    try {
      for (int i = 0; i < todoDataFromServer.length(); i++) {
        JSONObject jo = (JSONObject) todoDataFromServer.get(i);
        if (!(checkIfTaskExists(jo))) {
          insertTask(jo);  
        }
      }
    } catch (JSONException e) {
      Log.e(TAG, "JSONException: " + e);
    }
  }
  private boolean checkIfTaskExists(JSONObject jo) {
    String[] projection = new String[] {TodoDatabaseProvider.COL_TASK_ID, 
        TodoDatabaseProvider.COL_TASK };
    String selection = null;
    try {
      selection = TodoDatabaseProvider.COL_TASK + "='" 
                  + jo.getString(TodoDatabaseProvider.COL_TASK) + "'";
    } catch (JSONException e) {
      Log.e(TAG, "JSONException " + e);
      return false;
    }
    Cursor cursor = getContentResolver().query(TodoDatabaseProvider.CONTENT_URI, 
        projection, selection, null, null);
    if (cursor.getCount() == 0) {
      return false;
    } else {
      return true;
    }
  }
          
  private void insertTask(JSONObject jo) {
    Uri uri = getContentResolver().insert(TodoDatabaseProvider.CONTENT_URI, 
                                          null);
    if (uri == null) {
      Log.e(TAG, "Failed to insert new todo into " 
            + TodoDatabaseProvider.CONTENT_URI);
      finish();
      return;
    }
    setResult(RESULT_OK, (new Intent()).setAction(uri.toString()));
    ContentValues values = new ContentValues();
    try {
      values.put(TodoDatabaseProvider.COL_CREATEDATE, 
                 System.currentTimeMillis());
      values.put(TodoDatabaseProvider.COL_TASK, 
                 jo.getString(TodoDatabaseProvider.COL_TASK));
      values.put(TodoDatabaseProvider.COL_DUEDATE, 
                 jo.getString(TodoDatabaseProvider.COL_DUEDATE));
      values.put(TodoDatabaseProvider.COL_CATEGORY_LINK, 
                 jo.getInt(TodoDatabaseProvider.COL_CATEGORY_LINK));
    } catch (Exception e) {
      Log.e(TAG, "Exception: " + e.toString());
    }
    getContentResolver().update(uri, values, null, null);
  }
  
  private void getLocalTasks() {
    Cursor cursor = getContentResolver().query(
                    TodoDatabaseProvider.CONTENT_URI, 
                    localProjection, null, null, null);
    if (cursor.getCount() > 0) {
      cursor.moveToFirst();
      while (cursor.isAfterLast() == false) {
        String task = cursor.getString(cursor.
            getColumnIndex(TodoDatabaseProvider.COL_TASK));
        String duedate = cursor.getString(cursor.
            getColumnIndex(TodoDatabaseProvider.COL_DUEDATE));
        int createdate = cursor.getInt(cursor.
            getColumnIndex(TodoDatabaseProvider.COL_CREATEDATE));
        int category = cursor.getInt(cursor.
            getColumnIndex(TodoDatabaseProvider.COL_CATEGORY_LINK));
        JSONObject newTask = new JSONObject();
        try {
          newTask.put(TodoDatabaseProvider.COL_TASK, task);
          newTask.put(TodoDatabaseProvider.COL_DUEDATE, duedate);
          newTask.put(TodoDatabaseProvider.COL_CREATEDATE, createdate);
          newTask.put(TodoDatabaseProvider.COL_CATEGORY_LINK, category);
        } catch (JSONException e) {
          Log.e(TAG, "JSONException " + e);
          e.printStackTrace();
        }
        taskArray.put(newTask);
        cursor.moveToNext();
      }
    }
  }
    
  private void sendTasksToRemoteDb() {
    try {
      URL url = new URL(PUTADDRESS);
      HttpURLConnection urlConnection = (HttpURLConnection)url.openConnection();
      urlConnection.setDoOutput(true);
      urlConnection.setChunkedStreamingMode(0);
      urlConnection.setRequestMethod("GET");
      urlConnection.setRequestProperty("Content-Type", "application/json");
      DataOutputStream out = null;  
      out = new DataOutputStream(urlConnection.getOutputStream());
      out.writeBytes(taskArray.toString());
      out.flush();
      out.close();
      int status = ((HttpURLConnection) urlConnection).getResponseCode();
      urlConnection.disconnect();
    } catch (Exception e) {
      Log.e(TAG, "Exception: " + e);
      e.printStackTrace();
    }
  }
}
