package com.example.todo;

import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.DefaultHttpClient;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.app.ListActivity;
import android.content.ContentValues;
import android.content.Intent;
import android.database.Cursor;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;
import android.util.Log;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.Window;
import android.widget.SimpleCursorAdapter;
import android.widget.AdapterView.AdapterContextMenuInfo;

public class ToDo extends ListActivity {
  public static final String GETADDRESS = "http://www.example.com/test.php";
  public static final String PUTADDRESS = "http://www.example.com/test2.php";
  public static final int SERVER_UPDATE_ID = 0;
  private static final String TAG = "ToDo";

  @Override
  public void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);
      requestWindowFeature(Window.FEATURE_INDETERMINATE_PROGRESS);
      setContentView(R.layout.main);
      populateList();
      new SyncDataTask().execute(GETADDRESS, PUTADDRESS);
      /* This line is for when using RemoteTodoSync rather than 
       * SyncDataTask */
      /*startActivityForResult(new Intent(this.getBaseContext(),
                               RemoteTodoSync.class), SERVER_UPDATE_ID);*/
      registerForContextMenu(getListView());
  }

  @Override
  public void onResume() {
    super.onResume();
    populateList();
  }

  @Override
  public boolean onCreateOptionsMenu(Menu menu) {
    MenuInflater inflater = getMenuInflater();
    inflater.inflate(R.menu.todo_options_menu, menu);
    return super.onCreateOptionsMenu(menu);
  }

  @Override
  public void onCreateContextMenu(ContextMenu menu, View v, 
                                  ContextMenuInfo menuInfo) {
    super.onCreateContextMenu(menu, v, menuInfo);
    MenuInflater inflater = getMenuInflater();
    inflater.inflate(R.menu.todo_context_menu, menu);
  }

  @Override
  public boolean onOptionsItemSelected(MenuItem item) {
    switch (item.getItemId()) {
      case R.id.menu_add:
        Intent i = new Intent(Intent.ACTION_INSERT);
        i.setClass(this, TodoEditor.class);
        startActivity(i);
        return true;
      case R.id.menu_sync:
          /* This line is for when using RemoteTodoSync rather than 
         * SyncDataTask */
        /* startActivityForResult(new Intent(this.getBaseContext(), 
                                     RemoteTodoSync.class), SERVER_UPDATE_ID); */
          new SyncDataTask().execute(GETADDRESS, PUTADDRESS);
          return true;
      default:
        return super.onOptionsItemSelected(item);
    }
  }

  @Override
  public boolean onContextItemSelected(MenuItem item) {
    AdapterContextMenuInfo info = (AdapterContextMenuInfo) item.getMenuInfo();
    switch(item.getItemId()) {
      case R.id.menu_edit:
        Intent i = new Intent(Intent.ACTION_EDIT);
        i.setClass(this, TodoEditor.class);
        i.putExtra("id", info.id);
        startActivity(i);
        return true;
      case R.id.menu_delete:
        deleteItem(info.id);
        return true;
      default:
        return super.onContextItemSelected(item);
    }
  }
    
  /* This is used if calling RemoteTodoSync rather than using SyncDataTask */
  /*@Override
  protected void onActivityResult(int requestCode, int resultCode, Intent i) {
      super.onActivityResult(requestCode, resultCode, i);
      switch (requestCode) {
      case SERVER_UPDATE_ID:
          populateList();
      default:
          // do nothing; we don't expect any other results
      }
  }*/

  private void deleteItem(long id) {
    String where = TodoDatabaseProvider.COL_TASK_ID + "=" + id;
    getContentResolver().delete(TodoDatabaseProvider.CONTENT_URI, 
                                where, null);
    populateList();
  }

  private void populateList() {
        // CursorLoader recommended in API v11 onwards
    String joinQuery = "select " 
                       + TodoDatabaseProvider.TASK_TABLE + "." 
                       + TodoDatabaseProvider.ID + ", "
                       + TodoDatabaseProvider.TASK_TABLE + "." 
                       + TodoDatabaseProvider.COL_TASK + ", "
                       + TodoDatabaseProvider.TASK_TABLE + "." 
                       + TodoDatabaseProvider.COL_DUEDATE + ", "
                       + TodoDatabaseProvider.CATEGORY_TABLE + "." 
                       + TodoDatabaseProvider.COL_CATEGORY 
                       + " from " + TodoDatabaseProvider.TASK_TABLE + " join " 
                       + TodoDatabaseProvider.CATEGORY_TABLE + " on "
                       + TodoDatabaseProvider.TASK_TABLE + "."
                       + TodoDatabaseProvider.COL_CATEGORY_LINK + "="
                       + TodoDatabaseProvider.CATEGORY_TABLE + "." 
                       + TodoDatabaseProvider.ID; 
    Cursor cursor = TodoDatabaseProvider.getReadableDb().rawQuery(
                    joinQuery, null);

    String[] colNames = { TodoDatabaseProvider.COL_TASK, 
                          TodoDatabaseProvider.COL_DUEDATE, 
                          TodoDatabaseProvider.COL_CATEGORY };
    int[] viewIDs = { R.id.taskname, R.id.duedate, R.id.category };
    SimpleCursorAdapter adapter = new SimpleCursorAdapter(
        this,
        R.layout.todo_item,
        cursor,
        colNames,
        viewIDs
    );
    setListAdapter(adapter);
  }

  protected class SyncDataTask extends AsyncTask<String, CharSequence, 
                                                 Integer> {
    private static final String TAG = "SyncDataTask";
    //private final ProgressDialog dialog = new ProgressDialog(ToDo.this);
    private String[] localProjection = new String[] {
        TodoDatabaseProvider.COL_TASK,
        TodoDatabaseProvider.COL_DUEDATE,
        TodoDatabaseProvider.COL_CREATEDATE,
        TodoDatabaseProvider.COL_CATEGORY_LINK
    };        
    private String getAddress;
    private String putAddress;
    private JSONArray taskArray = new JSONArray();
        
    @Override
    protected void onPreExecute() {
//    publishProgress("starting to talk to server");
      ToDo.this.setProgressBarIndeterminateVisibility(true);
    }

    @Override
    protected Integer doInBackground(String... address) {
      getAddress = address[0];
      putAddress = address[1];
//    publishProgress("Fetching data from remote server");
      getData();
      addTodosFromServer(taskArray);
//    publishProgress("Sending data to remote server");
      getLocalTasks();
      sendTasksToRemoteDb();
      return RESULT_OK;
    }    
        
    @Override protected void onProgressUpdate(CharSequence... message) {
//     dialog.setMessage(message[0]);
//     dialog.show();
    }

    protected void onPostExecute(Integer result) {
//    dialog.dismiss();
      ToDo.this.setProgressBarIndeterminateVisibility(false);
      if (result == RESULT_OK) {
        populateList();
      }
    }
        
    private void getData() {
      InputStream is = null;
      try {
        HttpClient httpClient = new DefaultHttpClient();
        HttpPost httpPost = new HttpPost(getAddress);
        HttpResponse response = httpClient.execute(httpPost);
        HttpEntity entity = response.getEntity();
        is = entity.getContent();
      } catch (IOException e) {
        Log.e(TAG, "IOException " + e.toString());
        e.printStackTrace();
      }

      try {
        BufferedReader reader = new BufferedReader
            (new InputStreamReader(is, "iso-8859-1"), 8);
        String line = null;
        while ((line = reader.readLine()) != null) {
          taskArray = new JSONArray(line);
        }
        is.close();
      } catch (Exception e) {
        Log.e(TAG, "Exception: " + e.toString());
        e.printStackTrace();
      }
    }
        
    private void addTodosFromServer(JSONArray todoDataFromServer) {
      try {
        for (int i = 0; i < todoDataFromServer.length(); i++) {
          JSONObject jo = (JSONObject) todoDataFromServer.get(i);
          if (!(checkIfTaskExists(jo))) {
            insertTask(jo);    
          }
        }
      } catch (JSONException e) {
        Log.e(TAG, "JSONException: " + e);
      }
    }

    private boolean checkIfTaskExists(JSONObject jo) {
      String[] projection = new String[] {
          TodoDatabaseProvider.COL_TASK_ID, 
          TodoDatabaseProvider.COL_TASK };
      String selection = null;
      try {
         selection = TodoDatabaseProvider.COL_TASK + "='" 
                     + jo.getString(TodoDatabaseProvider.COL_TASK) + "'";
      } catch (JSONException e) {
        Log.e(TAG, "JSONException " + e);
        return false;
      }
      Cursor cursor = getContentResolver().query(
          TodoDatabaseProvider.CONTENT_URI, projection, selection, 
          null, null);
      if (cursor.getCount() == 0) {
        return false;
      } else {
        return true;
      }
    }
                    
    private void insertTask(JSONObject jo) {
      Uri uri = getContentResolver().insert(TodoDatabaseProvider.CONTENT_URI, 
                                            null);
      if (uri == null) {
        Log.e(TAG, "Failed to insert new todo into " 
              + TodoDatabaseProvider.CONTENT_URI);
        finish();
        return;
      }
      setResult(RESULT_OK, (new Intent()).setAction(uri.toString()));
      ContentValues values = new ContentValues();
      try {
        values.put(TodoDatabaseProvider.COL_CREATEDATE, 
                   System.currentTimeMillis());
        values.put(TodoDatabaseProvider.COL_TASK, 
                   jo.getString(TodoDatabaseProvider.COL_TASK));
        values.put(TodoDatabaseProvider.COL_DUEDATE, 
                   jo.getString(TodoDatabaseProvider.COL_DUEDATE));
        values.put(TodoDatabaseProvider.COL_CATEGORY_LINK, 
                   jo.getInt(TodoDatabaseProvider.COL_CATEGORY_LINK));
      } catch (Exception e) {
        Log.e(TAG, "Exception: " + e.toString());
      }
      getContentResolver().update(uri, values, null, null);
    }
    
    private void getLocalTasks() {
      TodoDatabaseProvider.setTable(TodoDatabaseProvider.TASK_TABLE);
      Cursor cursor = getContentResolver().query(
                      TodoDatabaseProvider.CONTENT_URI, 
                      localProjection, null, null, null);
      if (cursor.getCount() > 0) {
        cursor.moveToFirst();
        while (cursor.isAfterLast() == false) {
          String task = cursor.getString(cursor.
              getColumnIndex(TodoDatabaseProvider.COL_TASK));
          String duedate = cursor.getString(cursor.
              getColumnIndex(TodoDatabaseProvider.COL_DUEDATE));
          int createdate = cursor.getInt(cursor.
              getColumnIndex(TodoDatabaseProvider.COL_CREATEDATE));
          int category = cursor.getInt(cursor.
              getColumnIndex(TodoDatabaseProvider.COL_CATEGORY_LINK));
          JSONObject newTask = new JSONObject();
          try {
            newTask.put(TodoDatabaseProvider.COL_TASK, task);
            newTask.put(TodoDatabaseProvider.COL_DUEDATE, duedate);
            newTask.put(TodoDatabaseProvider.COL_CREATEDATE, createdate);
            newTask.put(TodoDatabaseProvider.COL_CATEGORY_LINK, category);
          } catch (JSONException e) {
            Log.e(TAG, "JSONException " + e);
            e.printStackTrace();
          }
          taskArray.put(newTask);
          cursor.moveToNext();
        }
      }
    }
    
    private void sendTasksToRemoteDb() {
      try {
        URL url = new URL(putAddress);
        HttpURLConnection urlConnection = 
            (HttpURLConnection)url.openConnection();
        urlConnection.setDoOutput(true);
        urlConnection.setChunkedStreamingMode(0);
        urlConnection.setRequestMethod("GET");
        urlConnection.setRequestProperty("Content-Type", "application/json");
        DataOutputStream out = null;    
        out = new DataOutputStream(urlConnection.getOutputStream());
        out.writeBytes(taskArray.toString());
        out.flush();
        out.close();
        int status = ((HttpURLConnection) urlConnection).getResponseCode();
        urlConnection.disconnect();
      } catch (Exception e) {
        Log.e(TAG, "Exception: " + e);
        e.printStackTrace();
      }
    }
  }
}

