package com.example.todo;

import android.app.ListActivity;
import android.content.Intent;
import android.database.Cursor;
import android.os.Bundle;
import android.util.Log;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.SimpleCursorAdapter;
import android.widget.AdapterView.AdapterContextMenuInfo;

public class ToDo extends ListActivity {
  private static final String TAG = "ToDo";

  /** Called when the activity is first created. */
  @Override
  public void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);
    setContentView(R.layout.main);
    populateList();
    registerForContextMenu(getListView());
  }

  @Override
  public void onResume() {
    super.onResume();
    populateList();
  }

  @Override
  public boolean onCreateOptionsMenu(Menu menu) {
    MenuInflater inflater = getMenuInflater();
    inflater.inflate(R.menu.todo_options_menu, menu);
    return super.onCreateOptionsMenu(menu);
  }

  @Override
  public void onCreateContextMenu(ContextMenu menu, View v, 
                                  ContextMenuInfo menuInfo) {
    super.onCreateContextMenu(menu, v, menuInfo);
    MenuInflater inflater = getMenuInflater();
    inflater.inflate(R.menu.todo_context_menu, menu);
  }

  @Override
  public boolean onOptionsItemSelected(MenuItem item) {
    switch (item.getItemId()) {
      case R.id.menu_add:
        Intent i = new Intent(Intent.ACTION_INSERT);
        i.setClass(this, TodoEditor.class);
        startActivity(i);
        return true;
      default:
        return super.onOptionsItemSelected(item);
    }
  }

  @Override
  public boolean onContextItemSelected(MenuItem item) {
    AdapterContextMenuInfo info = (AdapterContextMenuInfo) item.getMenuInfo();
    switch(item.getItemId()) {
      case R.id.menu_edit:
        Intent i = new Intent(Intent.ACTION_EDIT);
        i.setClass(this, TodoEditor.class);
        i.putExtra("id", info.id);
        startActivity(i);
        return true;
      case R.id.menu_delete:
        deleteItem(info.id);
        return true;
      default:
        return super.onContextItemSelected(item);
    }
  }

  private void deleteItem(long id) {
    String where = TodoDatabaseProvider.COL_TASK_ID + "=" + id;
    getContentResolver().delete(TodoDatabaseProvider.CONTENT_URI, 
              where, null);
    populateList();
  }

  private void populateList() {
    // CursorLoader recommended in API v11 onwards
    String joinQuery = "select " 
        + TodoDatabaseProvider.TASK_TABLE + "." 
        + TodoDatabaseProvider.ID + ", "
        + TodoDatabaseProvider.TASK_TABLE + "." 
        + TodoDatabaseProvider.COL_TASK + ", "
        + TodoDatabaseProvider.TASK_TABLE + "." 
        + TodoDatabaseProvider.COL_DUEDATE + ", "
        + TodoDatabaseProvider.CATEGORY_TABLE + "." 
        + TodoDatabaseProvider.COL_CATEGORY 
        + " from " + TodoDatabaseProvider.TASK_TABLE + " join " 
        + TodoDatabaseProvider.CATEGORY_TABLE + " on "
        + TodoDatabaseProvider.TASK_TABLE + "."
        + TodoDatabaseProvider.COL_CATEGORY_LINK + "="
        + TodoDatabaseProvider.CATEGORY_TABLE + "." + TodoDatabaseProvider.ID; 
    Cursor cursor = TodoDatabaseProvider.getReadableDb().
                    rawQuery(joinQuery, null);

    String[] colNames = { TodoDatabaseProvider.COL_TASK, 
                          TodoDatabaseProvider.COL_DUEDATE, 
                          TodoDatabaseProvider.COL_CATEGORY };
    int[] viewIDs = { R.id.taskname, R.id.duedate, R.id.category };
    SimpleCursorAdapter adapter = new SimpleCursorAdapter(
        this,
        R.layout.todo_item,
        cursor,
        colNames,
        viewIDs);
    setListAdapter(adapter);
  }
}

