package com.example.todo;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.ContentValues;
import android.content.ContentUris;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemSelectedListener;
import android.widget.EditText;
import android.widget.SimpleCursorAdapter;
import android.widget.Spinner;
import android.widget.TextView;


public class TodoEditor extends Activity {

  public static final String CLASSNAME = "TodoEditor";
  private static final String[] TODO_PROJECTION = new String[] {
      TodoDatabaseProvider.COL_TASK_ID, 
      TodoDatabaseProvider.COL_TASK,
      TodoDatabaseProvider.COL_DUEDATE,
      TodoDatabaseProvider.COL_CATEGORY_LINK
  };
  private static final String[] CATEGORY_PROJECTION = new String[] {
      TodoDatabaseProvider.COL_CATEGORY_ID, 
      TodoDatabaseProvider.COL_CATEGORY
  };
  private static final String INITIAL_TODO = "initialTodo";
  private static final int STATE_INSERT = 0;
  private static final int STATE_EDIT = 1;

  private static final String TAG = "TodoEditor";
  private int state;
  private Cursor cursor;
  private EditText text;
  private EditText date;
  private String initialTodo;
  private Uri uri;
  private int categorySelected;

  /** Called when the activity is first created. */
  @Override
  public void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);
        
    final Intent intent = getIntent();
    final String action = intent.getAction();
    if (Intent.ACTION_INSERT.equals(action)) {
      state = STATE_INSERT;
      uri = getContentResolver().insert(TodoDatabaseProvider.CONTENT_URI, null);
      if (uri == null) {
        Log.e(TAG, "Failed to insert new todo into " 
              + TodoDatabaseProvider.CONTENT_URI);
        finish();
        return;
      }
      setResult(RESULT_OK, (new Intent()).setAction(uri.toString()));
    } else if (Intent.ACTION_EDIT.equals(action)) {
      state = STATE_EDIT;
      long itemId = intent.getLongExtra("id", 0);
      uri = ContentUris.withAppendedId(TodoDatabaseProvider.CONTENT_URI, 
            itemId);
    } else {
      Log.e(TAG, "Unrecognised action " + action + ", exiting");
      finish();
      return;
    }
    TodoDatabaseProvider.setTable(TodoDatabaseProvider.TASK_TABLE);
    cursor = managedQuery(uri, TODO_PROJECTION, null, null, null);

    setContentView(R.layout.todo_editor);
    text = (EditText) findViewById(R.id.todo);
    date = (EditText) findViewById(R.id.duedate);

    Spinner category = (Spinner) findViewById(R.id.category);
    category.setOnItemSelectedListener(new CategoryItemSelectedListener());

    TodoDatabaseProvider.setTable(TodoDatabaseProvider.CATEGORY_TABLE);
    Cursor categoryCursor = managedQuery(TodoDatabaseProvider.CONTENT_URI, 
                            CATEGORY_PROJECTION, null, null, null);

    SimpleCursorAdapter adapter = new SimpleCursorAdapter(this,
        android.R.layout.simple_spinner_item,
        categoryCursor,
        new String[] {TodoDatabaseProvider.COL_CATEGORY},
        new int[] {android.R.id.text1} 
        );
    adapter.setDropDownViewResource(
        android.R.layout.simple_spinner_dropdown_item);
    category.setAdapter(adapter);
        
    if (state == STATE_EDIT) {
      cursor.moveToFirst();
      text.setText(cursor.getString(
                   cursor.getColumnIndex(TodoDatabaseProvider.COL_TASK)),
                   TextView.BufferType.EDITABLE);
      date.setText(cursor.getString(
                   cursor.getColumnIndex(TodoDatabaseProvider.COL_DUEDATE)), 
                   TextView.BufferType.EDITABLE);
      category.setSelection(cursor.getInt(
           		cursor.getColumnIndex(TodoDatabaseProvider.COL_CATEGORY_LINK))
          		- 1);
    }

    if (savedInstanceState != null) {
      initialTodo = savedInstanceState.getString(INITIAL_TODO);
    }
  }

  @Override
  public boolean onCreateOptionsMenu(Menu menu) {
    MenuInflater inflater = getMenuInflater();
    inflater.inflate(R.menu.editor_options_menu, menu);
    return super.onCreateOptionsMenu(menu);
  }

  @Override
  public boolean onOptionsItemSelected(MenuItem item) {
    switch (item.getItemId()) {
      case R.id.menu_save:
        String entryText = text.getText().toString();
        String entryDate = date.getText().toString();
        updateTodo(entryText, entryDate, categorySelected);
        finish();
        break;
      case R.id.menu_delete:
        deleteTodo(uri);
        finish();
        break;
      case R.id.menu_revert:
        cancelTodo();
        break;
    }
    return super.onOptionsItemSelected(item);
  }

  private final void updateTodo(String task, String duedate, int category) {
    ContentValues values = new ContentValues();
        
    if (state == STATE_INSERT || state == STATE_EDIT) {
      values.put(TodoDatabaseProvider.COL_CREATEDATE, 
                 System.currentTimeMillis());
      values.put(TodoDatabaseProvider.COL_TASK, task);
      values.put(TodoDatabaseProvider.COL_DUEDATE, duedate);
      values.put(TodoDatabaseProvider.COL_CATEGORY_LINK, category);
    }
    getContentResolver().update(uri, values, null, null);
  }

  private final void deleteTodo(Uri uri) {
    String where = TodoDatabaseProvider.COL_TASK_ID + "=" 
                   + ContentUris.parseId(uri);
	getContentResolver().delete(uri, where, null);
  	setResult(RESULT_OK, (new Intent()).setAction(uri.toString()));
  }
    
  private final void cancelTodo() {
  	if (state == STATE_INSERT) { deleteTodo(uri); }
   	setResult(RESULT_CANCELED, (new Intent()).setAction(uri.toString()));
  }

  public class CategoryItemSelectedListener implements OnItemSelectedListener {
    public void onItemSelected(AdapterView<?> parent, View v, int pos, 
                               long id) {
      Cursor c = (Cursor)parent.getSelectedItem();
      categorySelected = c.getInt(0);
    }

    public void onNothingSelected(AdapterView<?> parent) {
      // do nothing
    }
  }
}

